%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% C2boundary
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Reference:
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef C2boundary
    % Abstract class for C2-smooth closed boundary.
    
    properties
        points % coordinates of boundary points, an array of dimension 2 X nbPoints
        tvec % tangent vector
        tvec_norm % norm of the tangent vector
        avec % acceleration vector
        normal % outward normal vector
        nbPoints % number of boundary points
        sigma % element of curve integration.
    end
    
    methods
        function obj = C2boundary(points, tvec, avec)
            % Initialization of C2boundary object:
            % Inputs:
            % points: coordinates of boundary point, a 2 x nbPoints array
            % tvec: tangent vectors of boundary point, a 2 x nbPoints array
            % avec: acceleration vectors, a 2 x nbPoints array
            
            obj.points = points;
            obj.tvec = tvec;
            obj.avec = avec;
            
            normal = [0, 1; -1, 0]*tvec ; %orthogonal to tvec
            tt = sqrt(normal(1,:).^2+normal(2,:).^2);
            obj.normal = normal ./ repmat(tt,2,1); %normalized
            
            obj.nbPoints = size(points, 2);
            obj.tvec_norm = sqrt( obj.tvec(1,:).^2 + obj.tvec(2,:).^2 );
            obj.sigma = 2*pi / obj.nbPoints * obj.tvec_norm;
        end
        
        function plot(obj,varargin)
            % loop the curve
            X = [obj.points(1,:) , obj.points(1,1)];
            Y = [obj.points(2,:) , obj.points(2,1)];
            plot(X, Y, varargin{:});
        end
        
        function obj = lt(obj, phi)
            % Redefine the < operator as the rotation of the boundary
            if ~isa(phi, 'double') || length(phi) ~= 1
                error('Type error: only double scalar can be used for rotation.');
            end
            
            rot = [[cos(phi), -sin(phi)]; [sin(phi), cos(phi)]]; % Rotation matrix
            
            obj.points = rot * obj.points;
            obj.tvec = rot * obj.tvec;
            obj.normal = rot * obj.normal;
            obj.avec = rot * obj.avec;
        end
        
        function obj = mtimes(obj, s)
            % Overload of the operator *
            if ~isa(s, 'double') || length(s) ~= 1 || s <= 0
                error('Type error: only positive double scalar can be used for scaling.');
            end
            
            obj.points = obj.points * s;
            obj.tvec = obj.tvec * s;
            obj.tvec_norm = obj.tvec_norm*s;
            obj.avec = obj.avec * s;
            obj.sigma = obj.sigma*s;
        end
        
		function obj = plus(obj, z0)
			% Overload of the operator +
			if ~isa(z0, 'double')
				error('Type error: only double value can be used for translation.');
			end
			if size(z0,1) ~= 2 || size(z0,2) ~= 1
				error('Size error: translation vector must have size (2,1)');
			end
			obj.points = obj.points + repmat(z0,1,obj.nbPoints);
		end
    end
end

